/**
   @file
   @author Shin'ichiro Nakaoka
*/

#include "MultiSeqItem.h"
#include "Archive.h"
#include "PutPropertyFunction.h"
#include <boost/bind.hpp>
#include "gettext.h"

using namespace boost;
using namespace cnoid;


AbstractMultiSeqItem::AbstractMultiSeqItem()
{

}


AbstractMultiSeqItem::AbstractMultiSeqItem(const AbstractMultiSeqItem& org)
    : Item(org)
{

}


AbstractMultiSeqItem::~AbstractMultiSeqItem()
{

}


static bool setPropertyNumFrames(AbstractMultiSeqItem* item, int numFrames)
{
    if(numFrames >= 0){
        item->abstractSeq()->setNumFrames(numFrames);
        item->suggestFileUpdate();
        return true;
    }
    return false;
}


static bool setPropertyTimeLength(AbstractMultiSeqItem* item, double timeLength)
{
    if(timeLength >= 0){
        item->abstractSeq()->setTimeLength(timeLength);
        item->suggestFileUpdate();
        return true;
    }
    return false;
}


void AbstractMultiSeqItem::doPutProperties(PutPropertyFunction& putProperty)
{
    AbstractMultiSeqPtr seq = abstractSeq();

    putProperty(_("Frame rate"), seq->getFrameRate());
    putProperty(_("Num parts"), seq->getNumParts());
    putProperty(_("Num frames"), seq->getNumFrames(), bind(setPropertyNumFrames, this, _1));
    putProperty(_("Time length"), seq->getTimeLength(), bind(setPropertyTimeLength, this, _1));
    putProperty.decimals(3)(_("Time step"), seq->getTimeStep());
}


bool AbstractMultiSeqItem::store(Archive& archive)
{
    if(overwrite()){
        archive.writeRelocatablePath("filename", lastAccessedFilePath());
        archive.write("format", lastAccessedFileFormatId());
        return true;
    }
    return false;
}


bool AbstractMultiSeqItem::restore(const Archive& archive)
{
    std::string filename, formatId;
    if(archive.readRelocatablePath("filename", filename) && archive.read("format", formatId)){
        if(load(filename, formatId)){
            return true;
        }
    }
    return false;
}
