/**
   \file
   \author Shin'ichiro Nakaoka
*/

#include "OSGCollision.h"
#include <cnoid/OSGViewer>

#include <iostream>

using namespace std;
using namespace boost;
using namespace cnoid;

namespace {
    const bool TRACE_FUNCTIONS = false;
}


OSGCollision::OSGCollision()
{
    ppairs = 0;
    setSupportsDisplayList(false);
}


OSGCollision::OSGCollision(const OSGCollision& org, const osg::CopyOp& copyop)
    : osg::Drawable(org, copyop)
{
    ppairs = org.ppairs;
    setSupportsDisplayList(false);
}


/**
   \todo Use displaylists
*/
void OSGCollision::drawImplementation(osg::RenderInfo& ri) const
{
    OSGViewer* viewer = dynamic_cast<OSGViewer*>(ri.getView());

    if(viewer && !viewer->isCollisionVisibleMode()){
        return;
    }
    
    glPushAttrib(GL_ENABLE_BIT);
    glDisable(GL_LIGHTING);
    glColor3d(0.0, 1.0, 0.0);
    glBegin(GL_LINES);

    const std::vector<ColdetLinkPairPtr>& pairs = *ppairs;

    for(size_t i=0; i < pairs.size(); ++i){
        const std::vector<collision_data>& cols = pairs[i]->collisions();
        for(size_t j=0; j < cols.size(); ++j){
            const collision_data& col = cols[j];
            const Vector3 n = 50.0 * col.depth * col.n_vector;
            for(int k=0; k < col.num_of_i_points; ++k){
                if(col.i_point_new[k]){
                    const Vector3& p = col.i_points[k];
                    glVertex3dv(p.data());
                    glVertex3dv(Vector3(p + n).data());
                }
            }
        }
    }

    glEnd();
    glPopAttrib();
}


/**
   @todo improved the implementation.
*/
osg::BoundingBox OSGCollision::computeBound() const
{
    osg::BoundingBox bbox;
    return bbox;
}
